﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators {
  /// <summary>
  /// A base class for items that perform similarity calculation between two solutions.
  /// </summary>
  [Item("SimilarityCalculator", "A base class for items that perform similarity calculation between two solutions.")]
  [StorableClass]
  public abstract class SimilarityCalculator : Item, ISimilarityCalculator {
    #region Properties
    [Storable]
    private string target;
    public string Target {
      get { return target; }
      set { target = value; }
    }
    #endregion

    [StorableConstructor]
    protected SimilarityCalculator(bool deserializing) : base(deserializing) { }
    protected SimilarityCalculator(SimilarityCalculator original, Cloner cloner) : base(original, cloner) { }
    protected SimilarityCalculator() : base() { }

    public double ExecuteCalculation(IScope left, IScope right) {
      if (left == null || right == null)
        throw new ArgumentException("Cannot calculate diversity because one or both of the provided scopes is null.");

      if (left == right) return 1.0;
      else return CalculateSimilarity(left, right);
    }

    protected abstract double CalculateSimilarity(IScope left, IScope right);

    public override IDeepCloneable Clone(Cloner cloner) { return null; }
  }
}
